package com.tartare.sandbox.ejb;

import javax.annotation.Resource;
import javax.ejb.EJBContext;
import javax.ejb.Stateful;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jms.*;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.List;
import java.util.Collections;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: ewe
 * Date: 03.01.2008
 * Time: 16:09:57
 * To change this template use File | Settings | File Templates.
 */
@Stateful(
    name = "JmsTest",
    mappedName = "ejb/JmsTest"
)
        
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class JmsTestBean implements JmsTest {

    private final Logger LOGGER = Logger.getLogger(JmsTestBean.class.getName());

    @Resource
    EJBContext context;

    @Resource(mappedName = "jms/tartareConnectionFactory")
    public QueueConnectionFactory factory;

    @Resource(mappedName = "jms/tartareJobQueue")
    public Queue testQueue;


    public QueueConnection getConnection() {
        try {
            QueueConnection conn = factory.createQueueConnection();
            LOGGER.info("Obtained JMS connection from pool [factory=" + factory + "]");
            return conn;
        }
        catch (JMSException e) {
            LOGGER.log(Level.SEVERE, "Could not get JMS connection", e);
            throw new RuntimeException("Could not get JMS connection", e);
        }
    }


    public void closeConnection(QueueConnection conn) {
        try {
            if (conn != null) {
                conn.close();
                LOGGER.info("Released JMS connection back in pool [factory=" + factory + "]");

            }
        }
        catch (JMSException e) {
            LOGGER.log(Level.SEVERE, "Could not get JMS connection", e);
            throw new RuntimeException("Could not get JMS connection", e);
        }
    }

    public String join(List l) {
        return join(l.toArray());
    }

    public String join(List l, String delimiter) {
        return join(l.toArray(), delimiter);
    }

    public String join(Object[] o) {
        return join(o, ",");
    }

    public String join(Object[] o, String delimiter) {
        String s = "";

        for (int i = 0; i < o.length; i++) {
            if (o[i] == null) {
                s += "<null>";
            } else {
                s += o[i].toString();
            }

            if (i < (o.length - 1)) {
                s += delimiter;
            }
        }

        return s;
    }

    private static List globalJmsIds = Collections.synchronizedList(new ArrayList<String>());

      public void produceMessages( int nbMessageToProduce, int messageGroup ) throws Exception {

        QueueSession session = null;
        MessageProducer sender = null;
        QueueConnection conn = null;

        try {
            conn = getConnection();

            session = conn.createQueueSession( false, Session.AUTO_ACKNOWLEDGE );
            sender = session.createProducer(testQueue);

            for( int i=0; i<nbMessageToProduce; i++)
            {
                ObjectMessage msg = session.createObjectMessage ();
                msg.setObject ( new Integer( i ) );
                msg.setIntProperty("messageGroup", messageGroup);
                //sender.setPriority(messageGroup); // --> this will be a problem
                    
                sender.send (msg);
            }

            //LOGGER.info ("Messages sent (n="+nbMessageToProduce+")");

        }
        catch (Throwable e) {
            context.setRollbackOnly();
            LOGGER.log(Level.SEVERE, "Error when collection messages, rollback the transaction.", e);
        }
        finally {
            try {
                if (sender != null) {
                    sender.close();
                    //System.out.println("Sender closed.");
                }
                if (session != null) {
                    session.close();
                    //System.out.println("Session closed.");
                }

                closeConnection(conn);

            }
            catch (JMSException e2) {
                LOGGER.log(Level.FINEST, "Error when closing either the consumer, session or connection");
            }

        }

    }

    public void consumeMessages( int nbMessageToConsume, int messageGroup ) throws Exception {

        QueueSession session = null;
        //MessageConsumer consumer = null;
        QueueReceiver consumer = null;
        QueueConnection conn = null;

        try {
            conn = getConnection();

            // the arguments will probably be ignored, because behind the scenes,
            // the appserver is preparing an XA session
            session = conn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
            
            LOGGER.log(Level.FINEST, "Session was created");

            //consumer = session.createReceiver(testQueue);
            consumer = session.createReceiver(testQueue, "messageGroup=" + messageGroup);

            conn.start();
            LOGGER.log(Level.FINEST, "Delivery started");

            int collected = 0;
            boolean stop = false;

            List<String> jmsIds = new ArrayList<String>();

            while (!stop) {
                //Message collectedMessage = consumer.receive(1000);
                LOGGER.info(this + " is calling receive in group " + messageGroup);
                Message collectedMessage = consumer.receive();
                LOGGER.info(this + " has RETURNED from calling receive in group " + messageGroup);

                if (collectedMessage == null) {
                    LOGGER.log(Level.FINEST, "Timeout reached - no more messages in queue");
                    stop = true;
                } else {
                    //LOGGER.log(Level.FINEST, "Collected one message");

                    Object msg = ((ObjectMessage) collectedMessage).getObject();
                    String id = collectedMessage.getJMSMessageID().toString();
                    int actualMessageGroup = collectedMessage.getIntProperty("messageGroup");
                    //int actualMessageGroup = -99;
                    
                    LOGGER.info("Was expecting message in group " + messageGroup + ", got a message with group property: " +actualMessageGroup);
                    
                    jmsIds.add(id);

                    if (globalJmsIds.contains(id)) {
                        throw new Exception("*** JMS with ID=" + id + " already in list! ***");
                    } else {
                        globalJmsIds.add(id);
                    }
                    

                    collected++;
                    stop = (collected >= nbMessageToConsume);
                    
                }
            }

            LOGGER.log(Level.FINEST, "Delivery stopped");
            LOGGER.info("Collected messages for group:" + messageGroup + " : " + collected);
            LOGGER.info("Collected messages [" + join(jmsIds) + "]");

            //Thread.sleep(3 * 1000);

            LOGGER.info("Processing done [" + join(jmsIds) + "]");

        }
        catch (Exception e) {
            context.setRollbackOnly();
            LOGGER.log(Level.SEVERE, "Error when collection messages, rollback the transaction.", e);
            throw e;
        }
        finally {
            try {
                if (consumer != null) {
                    consumer.close();
                    System.out.println("Consumer closed.");
                }
                if (session != null) {
                    session.close();
                    System.out.println("Session closed.");
                }

                closeConnection(conn);

            }
            catch (JMSException e2) {
                LOGGER.log(Level.FINEST, "Error when closing either the consumer, session or connection");
            }

        }

    }
}
